
 /*+-------------------------------------------------------+
   |          SUGI 30 Sample                               |
   |          SAS/Graph: Tips and tricks                   |
   |                                                       |
   |    NAME: Annotate.sas                                 |
   |   TITLE: Create a Bar chart that shows pollen levels  |
   |          for each day.  Annotate labels, values and   |
   |          an image.  Placement specific to GIF         |
   | PRODUCT: GRAPH                                        |
   |                                                       |
   |   NOTES: Last Update 20Dec04                          |
   +-------------------------------------------------------+*/
%let name=annotate;
filename odsout '.';
goptions reset=all;

/*********************************************************************/
/* Get your data to be plotted.                                      */
/*********************************************************************/
data work.pollenbase;
   length pollentype $ 20 html $ 200;
   input day date7. pollen pollentype $ 19-39;

   /* Create data tips on the bars.                                  */
   /* Note: '0D'x should be replaced with blank when not using GIF   */
   html='alt='|| quote( 'Date: '||put(day,date9.)||'0D'x||
                 'Pollen Count: '||trim(left(pollen))||'0D'x||
                 'Pollen Type: '||pollentype||' ');
   format day downame.;
   format pollen comma4.1;
   cards;
09oct02 4.9 Ragweed 
10oct02 3.6 Ragweed
11oct02 2.0 Ragweed and Chenopod
12oct02 0.1 Ragweed and Chenopod
;
run;

data work.a; set work.pollenbase;run;

/*********************************************************************/
/* Annotate the number/value on the bar and date over bar.           */
/*********************************************************************/
data work.anno_a; 
   set work.a;
   length text $ 40 function $ 8 color style $ 12;
   retain position '5' when 'a' xsys '2' ysys '2';
   midpoint=day; 
   y=pollen;

   /* Draw white filled circle at top of each bar */
   function='pie';
   style='solid';
   color='white';
   rotate=360;
   size=2.25;
   output;

   /* Draw black ring around white circle at top of each bar */
   style='empty';
   color='black';
   output;

   /* Place numeric value in middle of white circle */
   function='label';
   y=y+.25; /*offset for the text*/
   color='red';
   style='"arial/bo"';
   size=2;
   text=trim(left(pollen));
   output;

   /* Put day of week name at top of graph, over each bar */
   function='label';
   y=14;
   color='black';
   style='"arial/bo"';
   size=2;
   text=trim(left(upcase(put(day,downame.))));
   output;

run;

/*********************************************************************/
/*  Annotate the side labels and image                               */
/*********************************************************************/
data work.other_anno;
   length  text $ 40 function $ 8 color style $ 12;
   retain function 'label' style '"arial/bo"' position '4' when 'a'
          xsys '3' ysys '3' size 2.5;
   x=11;
   y=55; color='red'; text='HIGH'; output;
   y=40; color='orange'; text='MED'; output;
   y=25; color='green'; text='LOW'; output;

   /* annotate the logo/image */
   function='move'; x=3; y=75; output;
   function='image'; x=87; y=90; imgpath='pollenbar.gif'; 
             style='fit'; output; 

run;

/* Combine all the annotate datasets*/
data work.anno_a; 
   set work.anno_a work.other_anno; 
run;


/* ---------------------------------------------------------*/     
/* Declare a macro to do the plot, for the specified device */
/* ---------------------------------------------------------*/     
%macro do_plot(dev);
   GOPTIONS DEVICE=&dev;
   ODS LISTING CLOSE;
   ODS HTML path=odsout body="&name._&dev..html"
            ;
   goptions border ftitle="arial/bo";

   /* color of bars */
   pattern1  c=red;

   /*space holder at top*/
   title1 j=l h=5.5 " ";
   title2 h=15pct " ";

   /* clear out the axis ticks, but leave space for annotate */
   /* and ref= lines                                         */
   axis1 order=(0 to 14 by 14) value=(tick=1 '' tick=2 '') minor=none 
         major=none style=0 label=none;
   axis2 label=(' ') color=blue width=7 
         value=(tick=1 '' tick=2 '' tick=3 '' tick=4 '');

   goptions hsize=7 vsize=4.1; /* propostions of graph */

   proc gchart data=work.a anno=work.anno_a; 
     vbar day / 
     html=html  /* cause data tips to show up on Bars and annotate*/
     discrete 
     width=12 /* width of bars */
     coutline=same  /* outline of bar same color as inside */
     raxis=axis1
     maxis=axis2
     /* draw several reflines close together (around 12, 8, and 4) */
     /* to make a "thick" refline                                  */
     ref=
         11.91 11.93 11.95 11.98 12 
          9 10 11
          7.95 7.98 8 8.02 8.05    
          5 6 7
          3.95 3.98 4 4.02 4.05    
          1 2 3    
      noframe
      sumvar=pollen 
      des='Pollen Level'
      name="&name"; 
   run;

   quit;
   ODS HTML CLOSE;
   ODS LISTING;
 %mend do_plot;

%do_plot(gif);


