
 /*+-------------------------------------------------------+
   |          SUGI 30 Sample                               |
   |          SAS/Graph: Tips and tricks                   |
   |                                                       |
   |    NAME: gtable.sas                                   |
   |   TITLE: Create a vertical bar chart with a graphical |
   |          table next to it. Placement specific to GIF  |
   | PRODUCT: GRAPH                                        |
   |                                                       |
   |   NOTES: Last Update 05Jan05                          |
   +-------------------------------------------------------+*/


%let name=gtable;
filename odsout '.';
goptions reset=all;


/**********************************************************************/
/* Get your election data to graph.  Calculate a margin.              */
/**********************************************************************/
data my_data;
   input st $ 1-2 winner $ 3-13 electoral_votes demo_votes repub_votes;

   margin=100*((repub_votes-demo_votes)/(repub_votes+demo_votes));
   abs_margin=abs(margin);
   state=stfips(st);
   datalines;
FL Republican 27 2912253 2912790
NM Democratic 5 289783 286418
WI Democratic 10 1242987 1237279
IA Democratic 7 638517 634373
OR Democratic 7 720342 713577
NH Republican 4 266348 273559
MN Democratic 10 1168266 1109659
MO Republican 11 1111138 1189924
NV Republican 5 279978 301575
OH Republican 20 2183628 2350363
TN Republican 11 981720 1061949
PA Democratic 21 2485967 2281127
ME Democratic 4 319951 286616
MI Democratic 17 2170418 1953139
AR Republican 6 422768 472940
WA Democratic 11 1247652 1108864
AZ Republican 10 685341 781652
WV Republican 5 295497 336475
;
run;
/**********************************************************************/
/* Sort the data by margin so it will increase down the grah          */
/**********************************************************************/
proc sort data=my_data out=my_data;
   by abs_margin;
run;
/**********************************************************************/
/* Add Datatips and drilldown to the data.  Also get state.           */
/**********************************************************************/
data my_data; 
   retain rankorder 100;
   length html $1000;
   set my_data;
   rankorder+1;
   statename=fipnamel(stfips(st));
   /*Data tips (alt=) and drilldown (href)                 */
   /*For devices other that GIF, replace '0D'x with a blank*/
   html='alt='|| quote(
            'State: '||trim(left(statename))||'0D'x||
            'Electoral Votes: '||trim(left(electoral_votes))||'0D'x||
            'Democratic = '||put(demo_votes,comma12.0)||'0D'x||
            'Republican = '||put(repub_votes,comma12.0)||'0D'x||
            'Margin: '||trim(left(put(abs_margin,3.1)))||'%  '
            ) ||' '||
        'href="http://www.princeton.edu/~rvdb/JAVA/election2000/"';
run;
/**********************************************************************/
/* Create an annotate dataset with the values for the table.          */
/* Color the text by winner.                                          */
/*********************************************************************/
data bar_anno1;
   set my_data;
   length function color cbox  $8 text $30;
   retain xsys ysys '2' when 'a' function 'label' size 1.25
          style '"arial/bo"' ;
   midpoint=rankorder;

   /* Output the percent value for the margin */
   position='6'; color='black';
   x=-8; text=trim(left(put(abs_margin,3.1))); output;

   /* Output the Republican value.  Red box with with white     */
   /* text if republican value bigger.  Else black text on white*/ 
   position='4';
   if (winner eq 'Republican') then do;
     cbox='cxb00026'; color='white'; /*red box, white text*/
	 statecol=cbox;
	 end;
   x=-9; text=trim(left(put(repub_votes,comma12.0))); output;

   /* Output the Democratic value.  Blue box with with white    */
   /* text if Democratic value bigger.  Else black text on white*/ 
   cbox='white'; color='black';
   if (winner eq 'Democratic') then do;
     cbox='cx0570b0'; color='white'; /*blue box, white text*/
	 statecol=cbox;
	 end;
   x=-16; text=trim(left(put(demo_votes,comma12.0))); output;

   /* Output the state name.  Color the text the value of winner*/
   position='6';
   cbox='white'; color=statecol;  
   x=-31; text=trim(left(statename)); output;
run;

/**********************************************************************/
/* Create an annotate dataset for labeling the columns.               */
/**********************************************************************/
data bar_anno2;
   length function color cbox  $8 text $30;
   retain xsys '2' ysys '3' when 'a' function 'label'
          size 1.25 style '"arial/bo"';

   y=90;
   color='GRAY';
   position='4';
   x=-9; text='Republican'; output;
   x=-16; text='Democratic'; output;
   x=-7; text='%'; output;

   /*write the header for the state names */
   position='6'; 
   x=-31; text='Closest States'; output;

   /*write the graph labels*/
   position='4'; x=-.2; color='cx0570b0'; text='Democrat'; output;
   position='6'; x=.2; color='cxb00026'; text='Republican'; output;
   y=93;
   position='5'; x=0; color='GRAY'; text='Margin of victory'; output;
run;

/* ----------------------------------------------------------*/     
/* Declare a macro to do the graph, for the specified device */
/* ----------------------------------------------------------*/     
%macro do_graph(dev);
   GOPTIONS DEVICE=&dev;
   ODS LISTING CLOSE;
   ODS HTML path=odsout body="&name._&dev..html"
            ;

   goptions border;
   goptions hsize=6in ftext="arial";

   title1 h=5pct "Barchart with a Table";
   title2 h=3pct " ";

   pattern1 v=s c=cx0570b0;  /* blue/democratic */
   pattern2 v=s c=cxb00026;  /* red/republican */

   axis1 c=black label=none value=none offset=(2,1);
   /* leave extra room on left to annotate table */
   axis2 c=black label=none value=none major=none 
         minor=none order=(-32 to 8 by 8);

   proc gchart data=my_data anno=bar_anno2;
      hbar rankorder /
           discrete
           anno=bar_anno1
           sumvar=margin
           subgroup=winner
           nostats
           noframe
           nolegend
           html=html
           maxis=axis1
           raxis=axis2
           des="" 
           name="&name"; 
   run;
   quit;

   ODS HTML CLOSE;
   ODS LISTING;
%mend do_graph;


%do_graph(gif);




