 /*+-------------------------------------------------------+
   |          SUGI 30 Sample                               |
   |          SAS/Graph: Tips and tricks                   |
   |                                                       |
   |    NAME: Spectrum.sas                                 |
   |   TITLE: Create a spectrum like graph that densely    |
   |          packs a lot of information.                  |
   | PRODUCT: GRAPH                                        |
   |                                                       |
   |   NOTES: Last Update 06JAN05                          |
   +-------------------------------------------------------+*/


%let name=spectrum;
filename odsout '.';
goptions reset=all;

%let mindate=01dec2003:00:00:00;
%let maxdate=08dec2003:00:00:00;

%let w=2.0;  /* Width of line segments */

/* Colors to use for bars and 'legend'*/
%let col1=CXFFFFB2;  
%let col2=CXFED976; 
%let col3=CXFEB24C;  
%let col4=CXFD8D3C;
%let col5=CXF03b20; 
%let col6=CXbd0026;   

/*********************************************************************/
/* Read in the data with the CPU usage.                              */
/*********************************************************************/
data cpuusage;
  format datetime datetime.;
  informat datetime datetime.;
  length hostname $ 8;
  infile 'cpuusage.dat';
  input datetime @18 hostname @27 totcpu;
  run;

/*********************************************************************/
/* Extract the hostnames (unique)                                    */
/*********************************************************************/
proc sql;
   create table goodname as select unique hostname 
          from cpuusage order by hostname;
   select count(hostname) into :count from goodname;
quit; 
run;

/*********************************************************************/
/* Number the hostnames.                                             */
/*********************************************************************/
data goodname; 
  set goodname;
  n+1;
run;

/*********************************************************************/
/* Merge the hostname numbers back into data.                        */
/*********************************************************************/
proc sql;
  create table cpuusage
         as select cpuusage.*, goodname.n
         from cpuusage join goodname
         on cpuusage.hostname = goodname.hostname;
quit; 
run;

/*********************************************************************/
/* Create an annotate dataset for hostname labels on left            */
/*********************************************************************/
data anno;
   set goodname;
   retain function 'label' xsys ysys '2' hsys '3' position '4'
          style '"arial/bo"' size 2.5;
   x="&mindate"dt;
   y=n-.5;
   text=trim(left(hostname))||" - ";
   run;

/*********************************************************************/
/* Add information for setting the color of bar and bar position.    */
/*********************************************************************/
data cpuusage;  
   set cpuusage;
   date = datepart(datetime);
   /*Q1, Q2, etc. in 'usage' is an ordered value for symbol/color*/
   usage='S1'; 
   if totcpu >  5 then usage='S2';
   if totcpu > 25 then usage='S3';
   if totcpu > 50 then usage='S4';
   if totcpu > 75 then usage='S5';
   if totcpu > 90 then usage='S6';
   /*y value for plot statement*/
   y=n-1+.03; output; /*offset around center position*/
   y=n-.03;   output;
   y=.;       output; /*disconnects line with 'skipmiss'*/

run;

/*********************************************************************/
/* Set up axis statements.                                           */
/*********************************************************************/
   /* Horizontal axis */
   axis1 label=none  
      order=("&mindate"DT to "&maxdate"DT by 86400) 
      minor=(number=3) 
      value=(angle=90)
      offset=(0,0)
      ;

   /* Vertical axis */
   axis2 label=(font="arial/bold" "      HOSTNAME  ")
      order=(0 to &count by 1)
      major=none minor=none 
      value=none
      offset=(0,0)
      ;


/* ---------------------------------------------------------*/     
/* Declare a macro to do the plot, for the specified device */
/* ---------------------------------------------------------*/     
%macro do_graph(dev);
   GOPTIONS DEVICE=&dev;
 
   ODS LISTING CLOSE;
   ODS HTML path=odsout body="&name._&dev..html"
       style=minimal
       ;

   goptions cback=white border;
   goptions ftitle="arial/bold" htitle=5pct ftext="arial" htext=2.5pct;
 
   title1 box=1 bcolor=black c=&col3 
           "Average Hourly CPU Usage (for one week)";

   /*Footnote is colored to replace legend*/
   footnote j=r box=1 bcolor=black  height=4pct font="arial/bold" 
            c=white  "Color Key:  "
            c=&col1  "0%-5%  "
            c=&col2  "5%-25%  "
            c=&col3  "25%-50%  "
            c=&col4  "50%-75%  "
            c=&col5  "75%-90%  "
            c=&col6  "90%-100%"
            ;

   /* symbols become the 6 color bars */
   symbol1  color=&col1  v=none interpol=join w=&w  l=1;
   symbol2  color=&col2  v=none interpol=join w=&w  l=1;
   symbol3  color=&col3  v=none interpol=join w=&w  l=1;
   symbol4  color=&col4  v=none interpol=join w=&w  l=1;
   symbol5  color=&col5  v=none interpol=join w=&w  l=1;
   symbol6  color=&col6  v=none interpol=join w=&w  l=1;

   proc gplot data=cpuusage; 
      plot y*datetime=usage / 
           skipmiss  /* skip the 'y' missing value*/
           autovref 
           vzero 
	       cframe=black /*black for tiny gaps between bars*/
           frame 
           nolegend    /* Footnote statement replaced it */
           haxis=axis1  
           vaxis=axis2 
           annotate=anno 
           des="" name="&name"
           ;
   run;
   quit;
   ODS HTML CLOSE;
   ODS LISTING;
%mend do_graph;
/*---------------------------------------------------------*/   

/*produce the graph for the device type*/
%do_graph(gif);

/*
%do_graph(javameta);
*/

