 /*+-------------------------------------------------------+
   |          SUGI 30 Sample                               |
   |          SAS/Graph: Tips and tricks                   |
   |                                                       |
   |    NAME: tally.sas                                    |
   |   TITLE: Create a graph that uses tally marks to      |
   |          indicate the 'bar' length. GIF device only   |
   | PRODUCT: GRAPH                                        |
   |                                                       |
   |   NOTES: Last Update 20Dec04                          |
   +-------------------------------------------------------+*/




%let name=tally;
filename odsout '.';
%let tickhght=6;
%let halfhght=3;
goptions reset;

/**********************************************************************/
/* Get your data with some basic value in it to be graphed            */
/**********************************************************************/
data mydata; 
  input name $ 1-10 count;
  cards;
Barbara    8
Cathy      15
Faith      26
Gail       31
Jill       17
Kelly      3
; 
run; 

/**********************************************************************/
/* Create an observation in the dataset for each value.  So a count of*/
/* '3' will have 3 observations, '10' will have 10 observations.      */
/* Each observation translates to a tally mark.                       */
/**********************************************************************/
data myanno; 
  set mydata;
  yc=name;
  /*create multiple observations for each value of count*/
  do xposn=1 to count;
    output;
  end;
run;


/**********************************************************************/
/* Create the tally marks in an annotate dataset.                     */
/* For each observation, create a tally mark based on it's position.  */
/* Create a mark across the others for every 5th value.               */
/**********************************************************************/
data myanno;
  length function color $ 8 ;
  retain xsys ysys '2' when 'a' size 3 color 'white';
 
  set myanno;

  /* first, move to the center of that tally tickmark coordinate */
  ysys='2';  /* absolute data coordinate */
  x=xposn;
  function='move'; 
  output;

  /* Draw the 4 vertical marks */
  if mod(xposn,5) ne 0 then do;
    type='norm';

    /* now, draw a little piece of line in the 'up' direction */
    ysys='7';  /* relative % of data area */ 
    function='draw'; 
    y=&halfhght; 
    output;

    /* now, extend the line 'down' from the center */
    y=-1*&tickhght; 
    output;
  end;

  /* Each 5th tally tickmark goes across the other 4 marks */
  else do;
    type='hash';
    /* Move cursor to top/right location of cross-hatch mark */
    /* move cursor 1/2-step to the left */
    function='move';   
    ysys='7';  /* relative % of data area */
    x=xposn-.5;

    /* move to top location of tally tickmark */
    y=&halfhght; 
    output;

    /* Now, draw line to bottom/left location of cross-hatch mark */
    function='draw';
    x=xposn-4.5;
    ysys='7';  /* relative % of data area */
    y=-1*&tickhght;
    output;
  end;
run;

/**********************************************************************/
/* Create the graph.                                                  */
/**********************************************************************/

/* Get rid of the axis */
axis1 offset=(5,5) label=none value=(justify=right);
axis2 offset=(0,0) label=none minor=none;

/* set the fonts, heights and colors for the text and background*/
goptions ftitle="arial" ftext="arial" gunit=pct htitle=6 
         htext=4 cback='LTGRAY';

/*standard ODS output*/
GOPTIONS DEVICE=gif;  /*or JAVAMETA*/
ODS LISTING CLOSE;
ODS HTML path=odsout body="&name._gif.html"
      style=minimal
            ;

/* Make the gplot symbol 'invisible' */  
symbol i=none v=dot c=ltgray h=.000000001;
    
title "Vote Tally Chart";
title2 "(created using GPLOT and Annotate)";
/* Add space around graph */
title3 h=2 a=90 " ";
title4 h=2 a=-90 " ";
footnote h=1.5 " ";

proc gplot data=mydata anno=myanno; 
     plot name*count / 
          vaxis=axis1
          haxis=axis2
          autohref chref=black
          noframe
          nolegend des="Vote Tally Chart" name="&name"; 
run; 

quit;
ODS HTML CLOSE;
ODS LISTING;
 
